﻿using Microsoft.AspNetCore.Mvc.ApplicationModels;
using Microsoft.Extensions.DependencyInjection;
using PoweredSoft.CQRS.Abstractions.Discovery;
using System;

namespace PoweredSoft.CQRS.AspNetCore.Mvc
{
    public class CommandControllerConvention : IControllerModelConvention
    {
        private readonly IServiceProvider serviceProvider;

        public CommandControllerConvention(IServiceProvider serviceProvider)
        {
            this.serviceProvider = serviceProvider;
        }

        public void Apply(ControllerModel controller)
        {
            if (!controller.ControllerType.IsGenericType)
                return;

            if (!controller.ControllerType.Name.Contains("CommandController"))
                return;
            
            if (controller.ControllerType.Assembly != typeof(CommandControllerConvention).Assembly)
                return;
                
            var genericType = controller.ControllerType.GenericTypeArguments[0];
            var commandDiscovery = this.serviceProvider.GetRequiredService<ICommandDiscovery>();
            var command = commandDiscovery.FindCommand(genericType);
            controller.ControllerName = command.LowerCamelCaseName;
        }
    }
}
